#-*- coding: utf-8 -*-

import tkinter as tk
from tkinter import ttk
import json

import drag_and_drop as DAD


class DADTreeComuni(DAD.DragDropElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def drop_data(self, descriptor, data: any) -> None:
        if not self.obj.dropable: return
        if descriptor != self.obj.tipe: return
        self.obj.set_comune(data)

    def drag_data(self) -> str:
        return self.obj.get_element()


class ElComuni(ttk.Treeview):
    def __init__(self, data: list, parent: callable, *args, **kwargs):
        super().__init__(parent, *args, **kwargs)
        self.parent = parent
        self.comuni = data
        self.configure(selectmode='browse')
        didas = ('Comune', 'Prov', 'istat', 'belfiore', 'sup. Kmq')
        self['columns'] = didas
        self['show'] = 'headings'
        for t in didas:
            self.heading(t, text=t)
        self.column('Prov', width=75, stretch=False)
        self.column('istat', width=100, stretch=False)
        self.column('belfiore', width=100, stretch=False)
        self.column('sup. Kmq', width=150, anchor=tk.E, stretch=False)
        self.__show_data()
        self.bind("<<TreeviewSelect>>", self.__on_select)

    def __show_data(self) -> None:
        for c in self.comuni:
            row = [c['den_ita'], c['prov'], c['istat'], c['belfiore'],
                   f"{c['sup']:0.4f}"]
            self.insert('', 'end', iid=None, values=row)

    def __on_select(self, evt: callable) -> None:
        iid = self.selection()[0]
        idx = self.index(iid)
        self.parent.set_comune(self.comuni[idx])

    def get_element(self) -> dict:
        iid = self.selection()[0]
        idx = self.index(iid)
        return self.comuni[idx]

class WinSearch(tk.Toplevel):
    ''' Simulazione di una finestra di ricerca. '''
    def __init__(self, parent: callable, *args, **kwargs):
        super().__init__(parent, *args, **kwargs)
        self.__populate()

    def __populate(self):
        self.title('Simulazione ricerca effettuata')
        lbl = tk.Label(self, text='Comune : ')
        lbl.grid(row=0, column=0, padx=(5,2), pady=5, sticky='w')
        self.e_comune = tk.Entry(self)
        self.e_comune.grid(row=0, column=1, padx=(2,5), pady=5, sticky='ew')
        DAD.DADEntry(obj=self.e_comune, tipe='name')
        self.e_comune.dropable = False
        lbl = tk.Label(self, text='CAP : ')
        lbl.grid(row=0, column=2, padx=(5,2), pady=5, sticky='w')
        self.cmb_cap = ttk.Combobox(self, width=6)
        self.cmb_cap.grid(row=0, column=2, padx=(2,5), pady=5, sticky='w')
        pn_data = tk.Frame(self)
        pn_data.grid(row=1, column=0, columnspan=4, sticky='ew')
        lbl = tk.Label(pn_data, text='Prov : ')
        lbl.grid(row=0, column=0, padx=(5,2), pady=5, sticky='w')
        self.e_prov = tk.Entry(pn_data, width=4)
        self.e_prov.grid(row=0, column=1, padx=(2,5), pady=5, sticky='w')
        DAD.DADEntry(obj=self.e_prov, tipe='provincia')
        self.e_prov.dropable = False
        lbl = tk.Label(pn_data, text='Cod. Istat : ')
        lbl.grid(row=0, column=2, padx=(5,2), pady=5, sticky='w')
        self.lbl_istat = tk.Label(pn_data, text='', width=8,
                                  relief='sunken', bg='white')
        self.lbl_istat.grid(row=0, column=3, padx=(2,5), pady=5, sticky='w')
        lbl = tk.Label(pn_data, text='Cod. Belfiore : ')
        lbl.grid(row=0, column=4, padx=(5,2), pady=5, sticky='w')
        self.e_belfiore = tk.Entry(pn_data, width=6)
        self.e_belfiore.grid(row=0, column=5, padx=(2,5), pady=5, sticky='w')
        DAD.DADEntry(obj=self.e_belfiore, tipe='belfiore')
        self.e_belfiore.dropable = False
        lbl = tk.Label(pn_data, text='Lat. : ')
        lbl.grid(row=0, column=6, padx=(5,2), pady=5, sticky='w')
        self.lbl_lat = tk.Label(pn_data, text='', width=10,
                                  relief='sunken', bg='white')
        self.lbl_lat.grid(row=0, column=7, padx=(2,5), pady=5, sticky='w')
        lbl = tk.Label(pn_data, text='Lon. : ')
        lbl.grid(row=0, column=8, padx=(5,2), pady=5, sticky='w')
        self.lbl_lon = tk.Label(pn_data, text='', width=10,
                                  relief='sunken', bg='white')
        self.lbl_lon.grid(row=0, column=9, padx=(2,5), pady=5, sticky='w')
        lbl = tk.Label(pn_data, text='Sup Kmq : ')
        lbl.grid(row=0, column=10, padx=(5,2), pady=5, sticky='w')
        self.lbl_sup = tk.Label(pn_data, text='', width=9,
                                  relief='sunken', bg='white')
        self.lbl_sup.grid(row=0, column=11, padx=(2,5), pady=5, sticky='w')
        pn_names = tk.LabelFrame(self, text=' Altre denominazioni : ')
        pn_names.grid(row=2, column=0, columnspan=4, padx=5, ipadx=5, pady=5, ipady=5, sticky='ew')
        lbl = tk.Label(pn_names, text='Italiana : ')
        lbl.grid(row=0, column=0, padx=(5,2), pady=5, sticky='w')
        self.e_den_ita = tk.Entry(pn_names)
        self.e_den_ita.grid(row=0, column=1, padx=(2,5), pady=5, sticky='ew')
        lbl = tk.Label(pn_names, text='Estranea : ')
        lbl.grid(row=1, column=0, padx=(5,2), pady=5, sticky='w')
        self.e_den_ext = tk.Entry(pn_names)
        self.e_den_ext.grid(row=1, column=1, padx=(2,5), pady=5, sticky='ew')
        pn_names.grid_columnconfigure(1, weight=1)
        self.__make_list()
        bt_close = tk.Button(self, text='Chiudi finestra', command=self.destroy)
        bt_close.grid(row=4, column=0, columnspan=4, padx=5, pady=5, sticky='ew')

        self.grid_rowconfigure(3, weight=1)
        self.grid_columnconfigure(1, weight=1)

    def __make_list(self) -> None:
        pn_data = tk.Frame(self)
        pn_data.set_comune = self.set_comune
        pn_data.grid(row=3, column=0, columnspan=4, padx=5, pady=5, sticky='nsew')
        fname = 'data_examples.json'
        with open(fname) as f:
            data = json.load(f)
        if not data or not len(data): return
        data.sort(key=lambda x: x['den_ita'])
        t_comuni = ElComuni(data, pn_data)
        t_comuni.grid(row=0, column=0, padx=(5,1), pady=5, sticky='nsew')
        vscr = tk.Scrollbar(pn_data, orient=tk.VERTICAL, command=t_comuni.yview)
        t_comuni.configure(yscrollcommand=vscr.set)
        vscr.grid(row=0, column=2, padx=(1,5), pady=5, sticky='ns')
        DADTreeComuni(obj=t_comuni, tipe='comune')
        t_comuni.dropable = False
        pn_data.grid_rowconfigure(0, weight=1)
        pn_data.grid_columnconfigure(0, weight=1)

    def set_comune(self, data: dict) -> None:
        self.e_comune.delete(0, tk.END)
        self.e_comune.insert(tk.END, data['den_ita'])
        if len(data['cod_p']) > 1:
            self.cmb_cap['values'] = [' '] + data['cod_p']
        else:
            self.cmb_cap['values'] = data['cod_p']
        self.cmb_cap.current(0)
        self.e_prov.delete(0, tk.END)
        self.e_prov.insert(tk.END, data['prov'])
        self.lbl_istat.configure(text=data['istat'])
        self.e_belfiore.delete(0, tk.END)
        self.e_belfiore.insert(tk.END, data['belfiore'])
        self.lbl_lat.configure(text=f"{data['lat']:0.6f}")
        self.lbl_lon.configure(text=f"{data['lon']:0.6f}")
        self.lbl_sup.configure(text=f"{data['sup']:0.4f}")
        self.e_den_ita.delete(0, tk.END)
        self.e_den_ita.insert(tk.END, data['den_ita'])
        self.e_den_ext.delete(0, tk.END)
        if data['den_altra']:
            self.e_den_ext.insert(tk.END, data['den_altra'])
        

class WinAddress(tk.Toplevel):
    def __init__(self, parent: callable, *args, **kwargs) -> None:
        super().__init__(parent, *args, **kwargs)
        self.parent = parent
        self._edit = False
        self.__populate()
        self.update()

    def __populate(self) -> None:
        self.title('Inserimento indirizzi')
        lbl = tk.Label(self, text='Via/P.zza : ')
        lbl.grid(row=0, column=0, padx=(5,2), pady=5, sticky='w')
        self.e_address = tk.Entry(self)
        self.e_address.grid(row=0, column=1, padx=(2,5), pady=5, sticky='ew')
        lbl = tk.Label(self, text='N° : ')
        lbl.grid(row=0, column=2, padx=(5,2), pady=5, sticky='w')
        self.e_num = tk.Entry(self, width=4)
        self.e_num.grid(row=0, column=3, padx=(2,5), pady=5, sticky='ew')
        self.pn_loc = tk.Frame(self)
        self.pn_loc.set_comune = self.set_comune
        DADTreeComuni(obj=self.pn_loc, tipe='comune')
        self.pn_loc.dropable = False
        self.pn_loc.dragable = False
        self.pn_loc.grid(row=1, column=0, columnspan=4, ipadx=3, ipady=3, sticky='ew')
        lbl = tk.Label(self.pn_loc, text='CAP : ')
        lbl.grid(row=0, column=0, padx=(5,2), pady=5, sticky='w')
        self.cmb_cap = ttk.Combobox(self.pn_loc, width=6)
        self.cmb_cap.grid(row=0, column=1, padx=(2,5), pady=5, sticky='w')
        lbl = tk.Label(self.pn_loc, text='Comune : ')
        lbl.grid(row=0, column=2, padx=(5,2), pady=5, sticky='w')
        self.e_comune = tk.Entry(self.pn_loc)
        self.e_comune.grid(row=0, column=3, padx=(2,5), pady=5, sticky='ew')
        DAD.DADEntry(obj=self.e_comune, tipe='name')
        self.e_comune.dropable = False
        self.e_comune.dragable = False
        lbl = tk.Label(self.pn_loc, text='Prov : ')
        lbl.grid(row=0, column=4, padx=(5,2), pady=5, sticky='w')
        self.e_prov = tk.Entry(self.pn_loc, width=4)
        self.e_prov.grid(row=0, column=5, padx=(2,5), pady=5, sticky='w')
        DAD.DADEntry(obj=self.e_prov, tipe='provincia')
        self.e_prov.dropable = False
        self.e_prov.dragable = False
        self.pn_loc.grid_columnconfigure(3, weight=1)
        pn_cmd = tk.Frame(self)
        pn_cmd.grid(row=2, column=0, columnspan=4, sticky='ew')
        self.bt_edit = tk.Button(pn_cmd, text='Modalità "Edit"')
        self.bt_edit.grid(row=0, column=0, sticky='nsew')
        bt_close = tk.Button(pn_cmd, text='Chiudi finestra',
                             command=self.destroy)
        bt_close.grid(row=0, column=1, sticky='nsew')
        pn_cmd.grid_columnconfigure(0, weight=1, uniform='bt')
        pn_cmd.grid_columnconfigure(1, weight=1, uniform='bt')
        self.grid_columnconfigure(1, weight=1)

        self.bt_edit.bind('<Button>', self.__on_edit)
        self.__on_edit(None)


    def __on_edit(self, evt) -> None:
        self._edit = not self._edit
        self.update()
        self.pn_loc.dropable = self._edit
        self.e_comune.dropable = self._edit
        self.e_prov.dropable = self._edit
        if self._edit:
            self.bt_edit.configure(text='Modalità "Static"')
        else:
            self.bt_edit.configure(text='Modalità "Edit"')

    def set_comune(self, data: dict) -> None:
        self.e_comune.delete(0, tk.END)
        self.e_comune.insert(tk.END, data['den_ita'])
        if len(data['cod_p']) > 1:
            self.cmb_cap['values'] = [' '] + data['cod_p']
        else:
            self.cmb_cap['values'] = data['cod_p']
        self.cmb_cap.current(0)
        self.e_prov.delete(0, tk.END)
        self.e_prov.insert(tk.END, data['prov'])
        
    
class MainWin(tk.Tk):
    def __init__(self):
        super().__init__()
        self.resizable(False, False)
        self.title('Finestra principale')
        bt_search = tk.Button(self, text='Finestra ricerca',
                              command=self._on_search)
        bt_search.grid(row=0, column=0, padx=5, pady=5, sticky='nsew')
        bt_address = tk.Button(self, text='Finestra indirizzo',
                               command=self._on_address)
        bt_address.grid(row=0, column=1, padx=5, pady=5, sticky='nsew')
        bt_close = tk.Button(self, text='Esci',
                               command=self._on_close)
        bt_close.grid(row=0, column=2, padx=5, pady=5, sticky='nsew')
        for i in range(3): self.grid_columnconfigure(i, weight=1, uniform='bt')

    def _on_search(self):
        win = WinSearch(self)
        
    def _on_address(self):
        win = WinAddress(self)
        
    def _on_close(self): self.destroy()



if __name__ == '__main__':
    app = MainWin()
    app.mainloop()
